import { CommonModule } from '@angular/common';
import { Injectable, Directive, ElementRef, Input, Renderer2, NgModule, defineInjectable, inject } from '@angular/core';
import { HttpErrorResponse, HttpResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { share, catchError, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CounterService {
    constructor() {
        this._connectionCounter = 0;
        this._observable = Observable.create((/**
         * @param {?} subscriber
         * @return {?}
         */
        subscriber => {
            this._subscriber = subscriber;
        })).pipe(share());
    }
    /**
     * @return {?}
     */
    get observable() {
        return this._observable;
    }
    /**
     * @return {?}
     */
    increment() {
        this._connectionCounter += 1;
        this._next();
    }
    /**
     * @return {?}
     */
    decrement() {
        this._connectionCounter -= 1;
        this._next();
    }
    /**
     * @private
     * @return {?}
     */
    _next() {
        if (this._subscriber) {
            this._subscriber.next(this._connectionCounter);
        }
    }
}
CounterService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
CounterService.ctorParameters = () => [];
/** @nocollapse */ CounterService.ngInjectableDef = defineInjectable({ factory: function CounterService_Factory() { return new CounterService(); }, token: CounterService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HttpClientBusyDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     * @param {?} counter
     */
    constructor(el, renderer, counter) {
        this.el = el;
        this.renderer = renderer;
        this.counter = counter;
        this.httpClientBusyConfig = {};
    }
    /**
     * @private
     * @param {?} count
     * @return {?}
     */
    _updateStatus(count) {
        /** @type {?} */
        const busy = count > 0;
        /** @type {?} */
        const hasIdle = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.idle);
        /** @type {?} */
        const hasBusy = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.busy);
        if (busy) {
            if (this.httpClientBusyConfig.idle && hasIdle) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
            if (this.httpClientBusyConfig.busy && !hasBusy) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
        }
        else {
            if (this.httpClientBusyConfig.busy && hasBusy) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
            if (this.httpClientBusyConfig.idle && !hasIdle) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.httpClientBusyConfig.idle = this.httpClientBusy.idle || 'fade';
        this.httpClientBusyConfig.busy = this.httpClientBusy.busy || '';
        this._updateStatus(0);
        this.autoUnsubscribe = this.counter.observable.subscribe((/**
         * @param {?} next
         * @return {?}
         */
        next => this._updateStatus(next)));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.autoUnsubscribe.unsubscribe();
    }
}
HttpClientBusyDirective.decorators = [
    { type: Directive, args: [{
                selector: '[httpClientBusy]'
            },] }
];
/** @nocollapse */
HttpClientBusyDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: CounterService }
];
HttpClientBusyDirective.propDecorators = {
    httpClientBusy: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HttpClientBusyInterceptor {
    /**
     * @param {?} counter
     */
    constructor(counter) {
        this.counter = counter;
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        this.counter.increment();
        return next.handle(req).pipe(tap((/**
         * @param {?} evt
         * @return {?}
         */
        evt => {
            if (evt instanceof HttpResponse) {
                this.counter.decrement();
            }
        })), catchError((/**
         * @param {?} err
         * @return {?}
         */
        err => {
            if (err instanceof HttpErrorResponse) {
                this.counter.decrement();
            }
            return throwError(err);
        })));
    }
}
HttpClientBusyInterceptor.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
HttpClientBusyInterceptor.ctorParameters = () => [
    { type: CounterService }
];
/** @nocollapse */ HttpClientBusyInterceptor.ngInjectableDef = defineInjectable({ factory: function HttpClientBusyInterceptor_Factory() { return new HttpClientBusyInterceptor(inject(CounterService)); }, token: HttpClientBusyInterceptor, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HttpClientBusyModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: HttpClientBusyModule
        };
    }
}
HttpClientBusyModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, HttpClientModule],
                declarations: [HttpClientBusyDirective],
                exports: [HttpClientBusyDirective],
                providers: [{ provide: HTTP_INTERCEPTORS, useClass: HttpClientBusyInterceptor, multi: true }]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CounterService, HttpClientBusyDirective, HttpClientBusyModule, HttpClientBusyInterceptor as ɵa };

//# sourceMappingURL=ngx-httpclient-busy.js.map