import { CommonModule } from '@angular/common';
import { Injectable, NgModule, Directive, ElementRef, Input, Renderer2, defineInjectable, inject } from '@angular/core';
import { HttpErrorResponse, HttpResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { share, catchError, tap } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CounterService = /** @class */ (function () {
    function CounterService() {
        var _this = this;
        this._connectionCounter = 0;
        this._observable = Observable.create((/**
         * @param {?} subscriber
         * @return {?}
         */
        function (subscriber) {
            _this._subscriber = subscriber;
        })).pipe(share());
    }
    Object.defineProperty(CounterService.prototype, "observable", {
        get: /**
         * @return {?}
         */
        function () {
            return this._observable;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CounterService.prototype.increment = /**
     * @return {?}
     */
    function () {
        this._connectionCounter += 1;
        this._next();
    };
    /**
     * @return {?}
     */
    CounterService.prototype.decrement = /**
     * @return {?}
     */
    function () {
        this._connectionCounter -= 1;
        this._next();
    };
    /**
     * @private
     * @return {?}
     */
    CounterService.prototype._next = /**
     * @private
     * @return {?}
     */
    function () {
        if (this._subscriber) {
            this._subscriber.next(this._connectionCounter);
        }
    };
    CounterService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    CounterService.ctorParameters = function () { return []; };
    /** @nocollapse */ CounterService.ngInjectableDef = defineInjectable({ factory: function CounterService_Factory() { return new CounterService(); }, token: CounterService, providedIn: "root" });
    return CounterService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var HttpClientBusyDirective = /** @class */ (function () {
    function HttpClientBusyDirective(el, renderer, counter) {
        this.el = el;
        this.renderer = renderer;
        this.counter = counter;
        this.httpClientBusyConfig = {};
    }
    /**
     * @private
     * @param {?} count
     * @return {?}
     */
    HttpClientBusyDirective.prototype._updateStatus = /**
     * @private
     * @param {?} count
     * @return {?}
     */
    function (count) {
        /** @type {?} */
        var busy = count > 0;
        /** @type {?} */
        var hasIdle = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.idle);
        /** @type {?} */
        var hasBusy = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.busy);
        if (busy) {
            if (this.httpClientBusyConfig.idle && hasIdle) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
            if (this.httpClientBusyConfig.busy && !hasBusy) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
        }
        else {
            if (this.httpClientBusyConfig.busy && hasBusy) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
            if (this.httpClientBusyConfig.idle && !hasIdle) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
        }
    };
    /**
     * @return {?}
     */
    HttpClientBusyDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.httpClientBusyConfig.idle = this.httpClientBusy.idle || 'fade';
        this.httpClientBusyConfig.busy = this.httpClientBusy.busy || '';
        this._updateStatus(0);
        this.autoUnsubscribe = this.counter.observable.subscribe((/**
         * @param {?} next
         * @return {?}
         */
        function (next) { return _this._updateStatus(next); }));
    };
    /**
     * @return {?}
     */
    HttpClientBusyDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.autoUnsubscribe.unsubscribe();
    };
    HttpClientBusyDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[httpClientBusy]'
                },] }
    ];
    /** @nocollapse */
    HttpClientBusyDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: CounterService }
    ]; };
    HttpClientBusyDirective.propDecorators = {
        httpClientBusy: [{ type: Input }]
    };
    return HttpClientBusyDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var HttpClientBusyInterceptor = /** @class */ (function () {
    function HttpClientBusyInterceptor(counter) {
        this.counter = counter;
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    HttpClientBusyInterceptor.prototype.intercept = /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    function (req, next) {
        var _this = this;
        this.counter.increment();
        return next.handle(req).pipe(tap((/**
         * @param {?} evt
         * @return {?}
         */
        function (evt) {
            if (evt instanceof HttpResponse) {
                _this.counter.decrement();
            }
        })), catchError((/**
         * @param {?} err
         * @return {?}
         */
        function (err) {
            if (err instanceof HttpErrorResponse) {
                _this.counter.decrement();
            }
            return throwError(err);
        })));
    };
    HttpClientBusyInterceptor.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    HttpClientBusyInterceptor.ctorParameters = function () { return [
        { type: CounterService }
    ]; };
    /** @nocollapse */ HttpClientBusyInterceptor.ngInjectableDef = defineInjectable({ factory: function HttpClientBusyInterceptor_Factory() { return new HttpClientBusyInterceptor(inject(CounterService)); }, token: HttpClientBusyInterceptor, providedIn: "root" });
    return HttpClientBusyInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var HttpClientBusyModule = /** @class */ (function () {
    function HttpClientBusyModule() {
    }
    /**
     * @return {?}
     */
    HttpClientBusyModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: HttpClientBusyModule
        };
    };
    HttpClientBusyModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, HttpClientModule],
                    declarations: [HttpClientBusyDirective],
                    exports: [HttpClientBusyDirective],
                    providers: [{ provide: HTTP_INTERCEPTORS, useClass: HttpClientBusyInterceptor, multi: true }]
                },] }
    ];
    return HttpClientBusyModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CounterService, HttpClientBusyDirective, HttpClientBusyModule, HttpClientBusyInterceptor as ɵa };

//# sourceMappingURL=ngx-httpclient-busy.js.map